/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcASCIIInputHandler.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */
#ifndef RTC_ASCII_INPUT_HANDLER_H
#define RTC_ASCII_INPUT_HANDLER_H

//== INCLUDES ==================================================================
#include "rtc/rtcInputHandler.h"

//== NAMESPACES ================================================================
namespace rtc {

/**
 * ASCII input handler.
 *
 * The ASCIIInputHandler class offers an interface for the functor
 * parameters and other classes to read() them in a ascii format.
 *
 * Example:
 * \code
 * // the ascii stream formatting object
 * rtc::ASCIIInputHandler bsh;
 *
 * // Read example
 *
 * // Open a stream in binary mode
 * std::ifstream in("testfile.txt");
 *
 * bsh.use(in);
 * int i;
 * rtc::read(bsh,i);
 *
 * std::string str;
 * rtc::read(bsh,str);
 *
 * in.close();
 * \endcode
 */
class ASCIIInputHandler : public InputHandler {
public:
  /// default constructor
  ASCIIInputHandler();
  /// construct from istream
  ASCIIInputHandler(std::istream& is);
  /// construct from filename
  ASCIIInputHandler(const char* filename);
  /// destructor
  virtual ~ASCIIInputHandler();
  /// returns if the data format is binary or ascii
  virtual bool binary() const;
  /// read the name of a name-data pair
  virtual bool readName(const std::string& name);
  /// read the separator of a name-data pair
  virtual bool readSeparator();
  //@{
  /**
   * read members for symbol-data pairs
   */
  virtual bool read(std::string& data);
  virtual bool read(double& data);
  virtual bool read(float& data);
  virtual bool read(int& data);
  virtual bool read(unsigned int& data);
  virtual bool read(char& data);
  virtual bool read(unsigned char& data);
  virtual bool read(bool& data);
  virtual bool read(long& data);
  virtual bool read(unsigned long& data);
  virtual bool read(short& data);
  virtual bool read(unsigned short& data);
  virtual bool read(char* data, long n);
  //@}
};

//==============================================================================
} // namespace rtc
//==============================================================================
#endif // RTC_ASCII_INPUT_HANDLER_H defined
//==============================================================================

