/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcOutputHandler.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */
#ifndef RTC_ASCII_OUTPUT_HANDLER_H
#define RTC_ASCII_OUTPUT_HANDLER_H

//== INCLUDES ==================================================================
#include "rtc/rtcOutputHandler.h"

//== NAMESPACES ================================================================
namespace rtc {

/**
 * Ascii output handler.
 *
 * The ASCIIOutputHandler class offers an interface for the functor
 * parameters and other classes to write() them in a
 * ascii format.
 *
 * Example:
 * \code
 * // the ascii stream formatting object
 * rtc::ASCIIOutputHandler bsh;
 *
 * // Write example:
 *
 * // open a stream in ascii mode
 * std::ofstream out("testfile.txt");
 *
 * // tell the ascii stream handler to use the given stream
 * bsh.use(out);
 *
 * rtc::write(bsh,5);
 * rtc::write(bsh,"hello world");
 *
 * out.close();
 * \endcode
 */
class ASCIIOutputHandler : public OutputHandler {
public:
  /// default constructor
  ASCIIOutputHandler();
  /// construct from ostream
  ASCIIOutputHandler(std::ostream& os);
  /// destructor
  virtual ~ASCIIOutputHandler();
  /// returns if the data format is binary or ascii
  virtual bool binary() const;
  /// write the name of a name-data pair
  virtual bool writeName(const std::string& name);
  /// write the separator of a name-data pair
  virtual bool writeSeparator();
  /// write the EOL symbol
  virtual bool writeEOL();
  //@{
  /**
   * write members for symbol-data pairs
   */
  virtual bool write(const std::string& data);
  virtual bool write(const double& data);
  virtual bool write(const float& data);
  virtual bool write(const int& data);
  virtual bool write(const unsigned int& data);
  virtual bool write(const char& data);
  virtual bool write(const unsigned char& data);
  virtual bool write(const bool& data);
  virtual bool write(const long& data);
  virtual bool write(const unsigned long& data);
  virtual bool write(const short& data);
  virtual bool write(const unsigned short& data);
  virtual bool write(const char* data, long n);
  //@}

protected:
  static const char SEPARATOR=' ';
};

//==============================================================================
} // namespace rtc
//==============================================================================
#endif // RTC_ASCII_OUTPUT_HANDLER_H defined
//==============================================================================

