/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcBase.h
 * authors ....: Benjamin Pitzer, Soeren Kammel
 * organization: Robert Bosch LLC
 * creation ...: 01/21/2009
 * modified ...: $Date: 2009-03-20 11:36:51 -0700 (Fri, 20 Mar 2009) $
 * changed by .: $Author: wg75pal $
 * revision ...: $Revision: 105 $
 */
#ifndef RTC_BASE_H_
#define RTC_BASE_H_

//== C Language Library Includes ===============================================
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <cctype>
#include <cstring>
#include <csignal>
#include <cmath>
#include <ctime>
#include <cerrno>
#include <climits>
#include <cfloat>
#include <assert.h>

#ifdef _MSC_VER
//# include <windows.h>
#   pragma warning(disable: 4100 4267 4311 4305)
typedef signed __int8 int8_t; 
typedef signed __int16 int16_t; 
typedef signed __int32 int32_t; 
typedef signed __int64 int64_t; 
typedef unsigned __int8 uint8_t; 
typedef unsigned __int16 uint16_t; 
typedef unsigned __int32 uint32_t; 
typedef unsigned __int64 uint64_t; 
#else
# include <unistd.h>
# include <sys/time.h>
# include <sys/types.h>
# include <sys/stat.h>
# include <stdint.h>		// Use the C99 official header
#endif

//== RTC Library Includes ======================================================
#include "rtc/rtcTime.h"
#include "rtc/rtcMisc.h"

#include "rtc/rtcException.h"
#include "rtc/rtcStringTools.h"

//== NAMESPACES ================================================================
namespace rtc {

// error, warning and verbose functions
#ifdef _MSC_VER
void rtc_perror(const char* fmt, ...);
void rtc_verbose(const char *fmt, ...);
void rtc_warn(const char* fmt, ...);
void rtc_die(const char* fmt, ...);
void rtc_die_syserror(const char* fmt, ...);
#else
void rtc_perror(const char* fmt, ...) __attribute__ ((format (printf, 1, 2)));
void rtc_verbose(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
void rtc_warn(const char* fmt, ...) __attribute__ ((format (printf, 1, 2)));
void rtc_die(const char* fmt, ...) __attribute__ ((format (printf, 1, 2)));
void rtc_die_syserror(const char* fmt, ...) __attribute__ ((format (printf, 1, 2)));
#endif
void rtc_carp_set_verbose(int verbosity);
int  rtc_carp_get_verbose(void);
void rtc_carp_set_output(FILE *output);

#define rtc_test_alloc(X) do {if ((void *)(X) == NULL) rtc_die("Out of memory in %s, (%s, line %d).\n", __FUNCTION__, __FILE__, __LINE__); } while (0)

// commandline parameters

// checks argv for occurence of "-param"
bool rtc_find_commandline_parameter(int argc, char **argv, const char* param);
// checks argv for occurence of "-param param_value" and returns param_value
bool rtc_find_commandline_parameter(int argc, char **argv, const char* param, char* param_value);
// displays a progress bar
void rtc_progress_bar_print(double current, double total);

// A macro to disallow the copy constructor and operator= functions
// This should be used in the private: declarations for a class
#define DISALLOW_COPY_AND_ASSIGN(TypeName) \
  TypeName(const TypeName&);               \
  void operator=(const TypeName&)

// math
#ifdef _MSC_VER
template <class T> inline bool isnan(T x) {
  return _isnan(x);
}
template <class T> inline bool isinf(T x) {
  return _isinf(x);
}
template <class T> inline T round(T x) {
  return floor(x+0.5);
}
#endif

//==============================================================================
} // namespace rtc
//==============================================================================
#endif // RTC_BASE_H_ defined
//==============================================================================
