#ifndef GRAPHICSHELPER_H
#define GRAPHICSHELPER_H

//== INCLUDES ==================================================================
#include <vector>
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "rtc/rtcGLInfo.h"
#include "rtc/rtcVec3.h"

//== NAMESPACES ================================================================
namespace rtc {

//== CLASS DEFINITION ==========================================================
class GraphicsHelper
{
public:
  GraphicsHelper(void);
  ~GraphicsHelper(void);

public:
  void drawSphere(double x, double y, double z, double scale=1.0);
  void drawCoordinateFrame(double scale = 1.0);
  void drawGrid(double center_x=0, double center_y=0, int size=100, double resolution=1.0) const;
  Vec3uc getColorFromPalette(int index);

protected:
  void initializeArrow();

  bool arrow_initialized;
  GLuint arrow_triangle_vertices_vbo;
  GLuint arrow_triangle_normals_vbo;
  GLuint arrow_quad_vertices_vbo;
  GLuint arrow_quad_normals_vbo;

  GLint initializeSphere();
  bool sphere_display_initialized;
  GLint sphere_display_list;
  GLuint sphere_vertices_vbo;
  GLuint sphere_normals_vbo;
  int num_sphere_vertices;

  static const GLbyte sphere_face_indicies[180][6];
  static const GLfloat sphere_vertices [92][3];
  static const GLfloat sphere_normals [102][3];

  #define COLOR_PALETTE_SIZE  13
  static const unsigned char graphics_colorpalette[COLOR_PALETTE_SIZE][3];
};

//==============================================================================
} // namespace rtc
//==============================================================================
#endif // GRAPHICSHELPER_H
//==============================================================================


