/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Mesh3DHelper.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 04/20/2008
 * modified ...: $Date:2008-03-03 10:26:02 -0800 (Mon, 03 Mar 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:141 $
 */
#ifndef MESH3DHELPER_H_
#define MESH3DHELPER_H_

//== INCLUDES ==================================================================
#include "rtc/rtcMesh3D.h"

//== NAMESPACES ================================================================
namespace rtc {

//== CLASS DEFINITION ==========================================================
class Mesh3DHelper
{
public:
  // standard c'tor and d'tor
  Mesh3DHelper();
  virtual ~Mesh3DHelper();

  // add noise
  static void addMeasurementNoise(Mesh3D& mesh, Vec3f& alpha_measurement);

  /**
   * update vertex normals
   *
   * This method uses a local point neighborhood and calculates the normal
   * by extracting principal directions. Face orientations are not considered
   * for this procedure.
   */
  static void updateVertexNormals(Mesh3D& srcdest, int n_neighbors=5);
  /**
   * remove isolated vertices
   *
   * This method removes isolated vertices which are not connected to
   * any triangle.
   */
  static void removeIsolatedVertices(Mesh3D& srcdest);
};

//==============================================================================
} // NAMESPACE rtc
//==============================================================================
#endif // MESH3DHELPER_H_ defined
//==============================================================================
