/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Mesh3DNode.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 04/16/2007
 * modified ...: $Date: 2009-09-10 13:26:19 -0700 (Thu, 10 Sep 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 915 $
 */
#ifndef MESH3DNODE_H
#define MESH3DNODE_H

//== INCLUDES ==================================================================
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "rtc/rtcPose3D.h"
#include "rtc/rtcMesh3D.h"
#include "rtc/rtcRenderNode.h"

//== NAMESPACES ================================================================
namespace rtc {

class Mesh3DNode : public RenderNode {
public:
  using RenderNode::setTransform;

  // default constructor
  Mesh3DNode(Renderer* renderer, Mesh3D* mesh);
  // destructor
  virtual ~Mesh3DNode();
  // initialize rendering
  virtual void initialize(Mesh3D* mesh);
  // clear rendering
  virtual void clear();
  // rendering call
  virtual void render();
  // prerendering call
  virtual void preRender();
  // postrendering call
  virtual void postRender();
  // seperate gpu loading process
  void loadToGPU();
  void unloadGPU();
protected:
  // draws mesh as faces in vertex buffer mode
  virtual void drawFacesVBO();
  // draws mesh as point cloud in vertex buffer mode
  virtual void drawPointsVBO();
  // draws mesh as wireframe in vertex buffer mode
  virtual void drawWireFrameVBO();
  // draws mesh as point cloud in immediate mode
  virtual void drawFaces();
  // draws mesh as point cloud in immediate mode
  virtual void drawPoints();
  // draws mesh as wireframe in immediate mode
  virtual void drawWireFrame();

  void bindVBO();
  void unbindVBO();
protected:
  bool has_texture;
  Texture texture;
  Image<Vec3uc> teximage;
  unsigned int num_vertices;
  unsigned int num_faces;

  unsigned int position_buffer;
  int position_size;
  GLfloat* position_data;

  unsigned int normal_buffer;
  int normal_size;
  GLfloat* normal_data;

  unsigned int color_buffer;
  int color_size;
  GLubyte* color_data;

  unsigned int texture_coordinate_buffer;
  int texture_coordinate_size;
  GLfloat* texture_coordinate_data;

  unsigned int index_buffer;
  int index_size;
  GLuint* index_data;
public:
  rtc::Pose3Df pose;
};

//==============================================================================
} // NAMESPACE puma
//==============================================================================
#endif // MESH3DNODE_H defined
//==============================================================================

