/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: MeshSet3D.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 02/29/2008
 * modified ...: $Date:2008-03-03 10:26:02 -0800 (Mon, 03 Mar 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:141 $
 */
#ifndef MESHSET3D_H
#define MESHSET3D_H

//== INCLUDES ==================================================================
#include <vector>
#include <iostream>
#include <rtc/rtcIOObject.h>
#include "rtc/rtcMesh3D.h"

//== NAMESPACES ================================================================
namespace rtc {
#define MESHSET3D_HAS_AABB 0x01

//== TYPE DECLARATIONS =========================================================
typedef std::vector<Mesh3D*> VecMesh3D;

//== CLASS DEFINITION ==========================================================
class MeshSet3D : public IOObject
{
public:
  using IOObject::writeToFile;
  using IOObject::readFromFile;

  // standard c'tor and d'tor
  MeshSet3D();
  MeshSet3D(const MeshSet3D& other);
  const MeshSet3D &operator=(const MeshSet3D& other);
  ~MeshSet3D();

  // Accessors
  int numMeshes() const;

  // Mutators
  void clear();
  void set(const MeshSet3D& other);
  // updates the normals in all meshes
  void updateNormals();

  // flags
  void setFlag(int flag, bool value);
  bool getFlag(int flag) const;
  bool hasAABB() const;

  // stream IO
  virtual bool write(OutputHandler &oh) const;
  virtual bool read(InputHandler &ih);
  void write(FILE *fp) const;
  void read(FILE *fp);
  bool writeToFile( const char* filename ) const;
  bool readFromFile( const char* filename );


  void loadToGPU();
  void unloadGPU();

  // Protected Data
  int flags;
  VecMesh3D meshes;
  Point3Df topleft,bottomright;
};

//==============================================================================
} // NAMESPACE rtc
//==============================================================================
#endif // MESHSET3D_H defined
//==============================================================================
