/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: MeshSet3DNode.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 09/04/2008
 * modified ...: $Date: 2009-09-10 13:26:19 -0700 (Thu, 10 Sep 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 915 $
 */
#ifndef MESHSET3DNODE_H
#define MESHSET3DNODE_H

//== INCLUDES ==================================================================
#define GL_GLEXT_PROTOTYPES
#include <GL/gl.h>
#include <GL/glu.h>
#include "rtc/rtcMeshSet3D.h"
#include "rtc/rtcRenderNode.h"
#include "rtc/rtcArray1.h"


//== NAMESPACES ================================================================
namespace rtc {
class Mesh3DNode;
class MeshSet3DNode : public RenderNode {
public:
  // default constructor
  MeshSet3DNode(Renderer* renderer, MeshSet3D* meshset);
  MeshSet3DNode(Renderer* renderer, const char* filename);
  // destructor
  virtual ~MeshSet3DNode();
  // initialize rendering
  virtual void initialize(MeshSet3D* meshset);
  virtual void initialize(const char* filename);
  // rebuild node
  virtual void rebuild(MeshSet3D* meshset);
  virtual void rebuild(const char* filename);
  void setupColorTable();
  void loadToGPU();
  void unloadGPU();

  // clear rendering
  virtual void clear();
  // rendering call
  virtual void render();
  // prerendering call
  virtual void preRender();
  // postrendering call
  virtual void postRender();
  // display faces
  virtual void displayFaces(bool b);
  // display vertices
  virtual void displayVertices(bool b);
  // display wireframe
  virtual void displayWireframe(bool b);
  // use vertex color
  virtual void setColorMode(Parameters::ColorMode b);
  // draws n-th mesh in a highlite color
  virtual void highliteMesh(int n);
  // get the n-th child
  virtual Mesh3DNode* getChild(int n);
  std::vector<Mesh3DNode*> nodes;
  Array1< Vec4<float> > ref_colors;
};

//==============================================================================
} // NAMESPACE puma
//==============================================================================
#endif // MESHSET3DNODE_H defined
//==============================================================================

