/*
* Copyright (C) 2008
* Robert Bosch LLC
* Research and Technology Center North America
* Palo Alto, California
*
* All rights reserved.
*
*------------------------------------------------------------------------------
* project ....: PUMA: Probablistic Unsupervised Model Acquisition
* file .......: MeshSet3DVBONode.h
* authors ....: Zhexi Wang
* organization: Robert Bosch LLC
* creation ...: 02/04/2009
* modified ...: $Date: 2009-02-02 12:17:10 -0800 (Mon, 02 Feb 2009) $
* changed by .: $Author: wg75pal $
* revision ...: $Revision: 772 $
*/
#ifndef MESHSET3DVBONODE_H
#define MESHSET3DVBONODE_H

#include <vector>
#include <GL/gl.h>
#include "rtc/rtcMeshSet3D.h"
#include "rtc/rtcRenderNode.h"

namespace rtc {
class MeshSet3DVBONode : public RenderNode
{
public:
  // default constructor
  MeshSet3DVBONode(Renderer* renderer, MeshSet3D* meshset);
  MeshSet3DVBONode(Renderer* renderer, const char* vbofilename);
  MeshSet3DVBONode(Renderer* renderer, FILE* fp);
  // destructor
  virtual ~MeshSet3DVBONode();
  // rendering call
  virtual void render();
  void loadToGPU();
  void unloadGPU();
  void writeToFile(const char *filename);
  void write(FILE* fp);

protected:
  // draws mesh as faces in vertex buffer mode
  virtual void drawFacesVBO();
  // draws mesh as point cloud in vertex buffer mode
  virtual void drawPointsVBO();
  // draws mesh as wireframe in vertex buffer mode
  virtual void drawWireFrameVBO();

protected:
public:
  int num_mesh;
  struct MeshVBO{
    unsigned int flag;
    unsigned int num_vertices;
    unsigned int num_faces;
    GLuint position_buffer;
    unsigned int position_size;
    GLfloat *position_data;
    GLuint normal_buffer;
    unsigned int normal_size;
    GLfloat *normal_data;
    GLuint color_buffer;
    unsigned int color_size;
    GLubyte *color_data;
    GLuint texcoord_buffer;
    unsigned int texcoord_size;
    GLfloat *texcoord_data;
    GLuint index_buffer;
    unsigned int index_size;
    GLuint *index_data;
    unsigned int texture_size;
    GLubyte *texture_data;
    unsigned int texture_width, texture_height;
    GLuint texture_id;
    ~MeshVBO();
    void read(FILE* fp);
    void write(FILE* fp);
    void loadGPU();
    void unloadGPU();
    void bind(unsigned int vbo);
    void unbind(unsigned int vbo);
  };
  MeshVBO *meshvbos;

  //!!!NO LOCK, SO MUST BE SYNCHRONIZED
  static std::vector<GLuint> gpubuffers;
  static std::vector<GLuint> gputextures;
  static void getGPUBuffer(GLuint &buffer);
  static void retGPUBuffer(GLuint &buffer);
  static void getTextureId(GLuint &texid);
  static void retTextureId(GLuint &texid);
};
}
#endif //MESHSET3DVBONODE_H
