/*------------------------------------------------------------------------------
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * description.:
 * authors ....: Soeren Kammel
 * organization: Robert Bosch LLC
 * creation ...: 01/26/2009
 * modified ...: $Date: 2009-02-16 17:39:28 -0800 (Mon, 16 Feb 2009) $
 * changed by .: $Author: kls1pal $
 * revision ...: $Revision: 48 $
 *------------------------------------------------------------------------------*/

#ifndef RTCBASE_MISC_H_
#define RTCBASE_MISC_H_

#include <cmath>

namespace rtc {

#define EPSILON  0.00001

	// check id close to zero
template <class T> inline bool fZero(T x) {return ( (x < EPSILON) && (x > -EPSILON) );}

	// round to closest integer
template <class T> inline int rInt(const T x) {return (x > 0 ? int(x+.5) : int(x-.5));}
//template <class T> inline T rInt(const T x) {return (x > 0 ? T(floor(x+.5)) : T(floor(x-.5)));}

	// round x to closest multiple of b; warning does not work if b is negative and x is positive
template <class T> inline T roundTo(const T x, const T b) {return b*T(floor(x/b+0.5));}

template <class T>
class rect {
public:
	rect() {}
	rect(T x_, T y_, T width_, T height_) : x(x_), y(y_), width(width_), height(height_) {}
	virtual ~rect() {}

T x, y;
T width, height;
};

typedef rect<unsigned int> rectui;
typedef rect<float> rectf;
typedef rect<double> rectd;

template <class T>
class size {
public:
	size() {}
	size(T width_, T height_) : width(width_), height(height_) {}
	virtual ~size() {}

T width, height;
};

typedef size<unsigned int> sizeui;
typedef size<float> sizef;
typedef size<double> sized;


template<class T, void*(T::*mem_fn)(void*)>
void* threadCBWrapper(void* ptr)
	{
	return (static_cast<T*>(ptr)->*mem_fn)(NULL);
	}

#ifdef __DARWIN__
inline void pthread_yield() {pthread_yield_np();}
#endif

}	// namespace rtc

#endif // RTCBASE_MISC_H_
