/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Point3DKdTree.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 10/29/2008
 * modified ...: $Date:2008-03-03 10:26:02 -0800 (Mon, 03 Mar 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:141 $
 */
#ifndef POINT3DKDTREE_H_
#define POINT3DKDTREE_H_

//== INCLUDES ==================================================================
#include "ANN/ANN.h"
#include "rtc/rtcVec3.h"

//== NAMESPACES ================================================================
namespace rtc {

//== FORWARD DECLARATIONS ======================================================

//== CLASS DEFINITION ==========================================================
class Point3DKdTree
{
public:
  Point3DKdTree(float* x, float* y, float* z, int num_points);
  virtual ~Point3DKdTree();
  // find closest point in kd-tree, returns index to this point
  int findNearest(const Vec3f& p);
  int findNearest(float x, float y, float z);
  // find closest k points in kd-tree
  void findNearest(float x, float y, float z, int k, int* points);
  void findNearest(float x, float y, float z, int k, std::vector<int>& points);
  void findNearest(const Vec3f& p, int k, int* points);
  void findNearest(const Vec3f& p, int k, std::vector<int>& points);

  // find all points within range
  int findWithinRange(float x, float y, float z, float range, std::vector<int>& points);
  int findWithinRange(const Vec3f& p, float range, std::vector<int>& points);
protected:
  // internal search method
  void findNearest(float* ann_sample, ANNidx* i, ANNdist* d);
  // KD-Tree data
  ANNkd_tree *ann_tree;
  ANNpointArray dataPts;
};

//==============================================================================
} // NAMESPACE puma
//==============================================================================
#endif // POINT2DKDTREE_H_ defined
//==============================================================================
