#ifndef RENDERER_MANAGER_H
#define RENDERER_MANAGER_H

//== INCLUDES ==================================================================
#define GL_GLEXT_PROTOTYPES
#include <GL/glew.h>
#include <GL/glui.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <list>
#include "rtc/rtcRenderer.h"

//== CONSTANTS =================================================================
#define GL_GLEXT_PROTOTYPES
#define MAX_NUMBER_OF_RENDERERS 256

//== NAMESPACES ================================================================
namespace rtc {

class Renderer;
class RenderManager {

public:
  /**
  * default constructor
  */
  RenderManager(int* argc, char **argv);

  /**
  * destructor
  */
  virtual ~RenderManager();

  /**
  * create a rendering window
  */
  int createWindow(const char* title, Renderer* renderer);

  /**
  * rendering mainloop
  */
  void  dispatch(void);

  void  disableIdleFunction(void);
  void  enableIdleFunction(void);
  int   idleFunctionEnabled(void);
  int   idleSetToCurrentWindow(void);
  void  setIdleToCurrentWindow(void);

private:
  static void callbackDisplayFunc(void);
  static void callbackIdleFunc(void);
  static void callbackKeyboardFunc(unsigned char key, int x, int y);
  static void callbackMotionFunc(int x, int y);
  static void callbackMouseFunc(int button, int state, int x, int y);
  static void callbackPassiveMotionFunc(int x, int y);
  static void callbackReshapeFunc(int w, int h);
  static void callbackSpecialFunc(int key, int x, int y);
  static void callbackVisibilityFunc(int visible);
  static void callbackTimerFunc(int value);

  static int m_currentIdleRenderer;
  static int m_idleFunctionEnabled;
};

//==============================================================================
} // namespace rtc
//==============================================================================
#endif // RENDERER_MANAGER_H defined
//==============================================================================

