/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: RenderNode.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 04/16/2007
 * modified ...: $Date: 2009-02-10 21:31:50 -0800 (Tue, 10 Feb 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 801 $
 */
#ifndef RENDER_NODE_H
#define RENDER_NODE_H

//== INCLUDES ==================================================================
#include "rtc/rtcTransform.h"

//== NAMESPACES ================================================================
namespace rtc {

/**
* This type defines the mode for rendering
*/
enum eRenderMode {
  Immediate = 0,
  VertexBufferObjects = 1,
  DisplayLists = 2,
};

class Renderer;
class RenderNode {

public:
  /**
  * the Parameters for the class Renderer
  */
  class Parameters {
  public:
  /**
  * default constructor
  */
  Parameters();

  /**
  * copy constructor
  * @param other the Parameters object to be copied
  */
  Parameters(const Parameters& other);

  /**
  * destructor
  */
    virtual ~Parameters();

    /**
    * copy the contents of a Parameters object
    * @param other the Parameters object to be copied
    * @return a reference to this Parameters object
    */
    Parameters& copy(const Parameters& other);

    /**
    * copy the contents of a Parameters object
    * @param other the Parameters object to be copied
    * @return a reference to this Parameters object
    */
    Parameters& operator=(const Parameters& other);


    /**
    * returns a pointer to a clone of the Parameters
    */
    virtual RenderNode::Parameters* clone() const;

    /**
    * read the parameters
    */
    virtual bool write(std::ostream& os) const;

    /**
    * read the parameters
    */
    virtual bool read(std::istream& is);

    // ------------------------------------------------
    // the Parameters
    // ------------------------------------------------
    bool draw_faces;
    bool draw_vertices;
    bool draw_wireframe;
    bool use_vertex_colors;
    typedef enum {NO_COLOR,VERTEX_COLOR,TEXTURE_COLOR} ColorMode;
    ColorMode color_mode;
    bool highlite;
  };

  // default constructor
  RenderNode(Renderer* renderer);
  // destructor
  virtual ~RenderNode();
  // Set Renderers's Parameters.
  virtual bool setParameters(const Parameters& theParam);
  // returns used Parameters
  const Parameters& getParameters() const;
  // returns used Parameters
  Parameters& getParameters();
  // initialize rendering
  virtual void initialize();
  // rebuild node
  virtual void rebuild();
  // clear rendering
  virtual void clear();
  // rendering call
  virtual void render()=0;
  // prerendering call
  virtual void preRender();
  // post rendering call
  virtual void postRender();
  //returns the render mode
  eRenderMode getRenderMode();
  // sets transform for render node
  void setTransform(Transformf transform);
  //returns the render node's transform
  const Transformf& getTransform();
  // display faces
  virtual void displayFaces(bool b);
  // display vertices
  virtual void displayVertices(bool b);
  // display wireframe
  virtual void displayWireframe(bool b);
  // set color mode
  virtual void setColorMode(Parameters::ColorMode b);
  // draws in a highlite color
  virtual void setHighlite(bool b);
protected:
  /**
  * current Parameters.
  */
  Parameters* m_params;

  /**
  * initialized flag
  */
  bool m_is_initialized;

  /**
  * pointer to the renderer
  */
  Renderer* m_renderer;

  /**
  * transform for this node
  */
  Transformf m_transform;

};

//=============================================================================
} // NAMESPACE rtc
//=============================================================================
#endif // RENDER_NODE_H defined
//=============================================================================

