/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Texture.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 10/13/2006
 * modified ...: $Date: 2009-01-29 16:51:00 -0800 (Thu, 29 Jan 2009) $
 * changed by .: $Author: wg75pal $
 * revision ...: $Revision: 771 $
 */
#ifndef TEXTURE_H
#define TEXTURE_H

//== INCLUDES ==================================================================
#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <rtc/rtcImage.h>

//== NAMESPACES ================================================================
namespace rtc {

//== CLASS DEFINITION ==========================================================
class Texture {
public:
  // c'tor
  Texture(bool multi_use=false, int max_texture_size=1024);
  // d'tor
  virtual ~Texture();

  // create empty texture
  void create(int width, int height);
  // update texture
  void update();
  // create empty RGBA texture
  void createRGBA(int width, int height);
  // update RGBA texture
  void updateRGBA();

  // draw texture
  void draw();
  void draw2(double x1, double y1, double x2, double y2,
             double x3, double y3, double x4, double y4);

  // load texture from image
  bool loadFromImage(const char *filename);
  bool saveToImage(const char* filename);
  void unloadTexture();
  // create texture from image
  void fromImage(const Image4uc& image);
  void fromImage(const Image3uc& image);
  void toImage(Image4uc& image);

  // bind/unbind
  void bind(GLenum target=GL_TEXTURE0);
  void unbind();

  // stream IO
  virtual bool write(std::ostream& os, bool ascii) const;
  virtual bool read(std::istream& is, bool ascii);
public:
  bool multi_use;
  int max_texture_size;
  unsigned int texture_id;
  int texture_width;
  int texture_height;
  int image_width;
  int image_height;
  float max_u, max_v;
  unsigned char *texture_data;

  GLenum bind_target;
};

//==============================================================================
} // namespace rtc
//==============================================================================
#endif //TEXTURE_H
