/*
 * Copyright (C) 2007
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: mesh.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 8/4/2006
 * modified ...: $Date: 2009-08-25 17:32:44 -0700 (Tue, 25 Aug 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 893 $
 */
#ifndef MESH_H
#define MESH_H

//== INCLUDES ==================================================================
#include <rtc/rtcVec3.h>
#include <rtc/rtcMesh3D.h>
#include <OpenMesh/Core/IO/MeshIO.hh>
#include <OpenMesh/Core/Mesh/TriMesh_ArrayKernelT.hh>

//== NAMESPACES ================================================================
namespace rtc {

  // Define the mesh
  struct TriMeshTraits : public OpenMesh::DefaultTraits
  {
    VertexAttributes   (OpenMesh::Attributes::Normal |
                        OpenMesh::Attributes::Color |
                        OpenMesh::Attributes::Status);
    HalfedgeAttributes (OpenMesh::Attributes::PrevHalfedge);
    FaceAttributes     (OpenMesh::Attributes::Normal);
  };
  typedef OpenMesh::TriMesh_ArrayKernelT<TriMeshTraits>  TriMesh;

  // face stuff
  typedef TriMesh::FaceHandle                            FaceHandle;
  typedef TriMesh::FaceIter                              FaceIter;
  typedef TriMesh::ConstFaceIter                         ConstFaceIter;
  typedef TriMesh::ConstFaceFaceIter                     ConstFaceFaceIter;
  typedef std::vector<FaceHandle>                        FHandles;
  typedef OpenMesh::FPropHandleT<int>                    FPropInt;
  typedef TriMesh::FaceVertexIter                        FaceVertexIter;
  typedef TriMesh::ConstFaceVertexIter                   ConstFaceVertexIter;

  // vertex stuff
  typedef TriMesh::VertexHandle                          VertexHandle;
  typedef TriMesh::VertexIter                            VertexIter;
  typedef TriMesh::VertexVertexIter                      VertexVertexIter;
  typedef TriMesh::ConstVertexIter                       ConstVertexIter;
  typedef TriMesh::ConstVertexVertexIter                 ConstVertexVertexIter;
  typedef std::vector<VertexHandle>                      VHandles;
  typedef OpenMesh::VPropHandleT<int>                    VPropInt;

  // edge stuff
  typedef TriMesh::EdgeHandle                            EdgeHandle;
  typedef TriMesh::EdgeIter                              EdgeIter;
  typedef TriMesh::ConstEdgeIter                         ConstEdgeIter;
  typedef TriMesh::HalfedgeHandle                        HalfedgeHandle;
  typedef OpenMesh::EPropHandleT<double>                 EPropDouble;

  // other stuff
  typedef TriMesh::Point                                 Point;
  typedef TriMesh::Color                                 Color;
  typedef TriMesh::Normal                                Normal;

  // Handle for a texture entity
  struct TextureHandle : public OpenMesh::BaseHandle
  {
    explicit TextureHandle(int _idx=-1) : BaseHandle(_idx) {}
  };

  // helper functions
  Vec3f point_to_vec(const Point& p);
  Vec3f normal_to_vec(const Normal& p);
  Vec3uc color_to_vec(const Color& p);
  Point vec_to_point(const Vec3f& v);
  Normal vec_to_normal(const Vec3f& v);
  Color vec_to_color(const Vec3uc& v);
  void convertMeshToTriMesh(const Mesh3D& input, TriMesh& output);
  void convertTriMeshToMesh(const TriMesh& input, Mesh3D& output);

//==============================================================================
} // NAMESPACE puma
//==============================================================================
#endif // MESH_H defined
//==============================================================================
