/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Vertex3D.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 02/29/2008
 * modified ...: $Date:2008-03-03 10:26:02 -0800 (Mon, 03 Mar 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:141 $
 */
#ifndef VERTEX3D_H_
#define VERTEX3D_H_

//== INCLUDES ==================================================================
#include <vector>
#include <rtc/rtcBase.h>
#include <rtc/rtcIOObject.h>
#include <rtc/rtcPoint3D.h>
#include <rtc/rtcVec2.h>

//== NAMESPACES ================================================================
namespace rtc {

//== FORWARD DECLARATIONS ======================================================
class Mesh3D;
class Face3D;

#define VERTEX_FLAG_BOUNDARY          0x01
#define VERTEX_FLAG_HIDDEN            0x02
#define VERTEX_FLAG_HAS_TEX_COORD     0x04
#define VERTEX_FLAG_EMPTY3            0x08
#define VERTEX_FLAG_EMPTY4            0x10
#define VERTEX_FLAG_EMPTY5            0x20
#define VERTEX_FLAG_EMPTY6            0x40
#define VERTEX_FLAG_EMPTY7            0x80


//== CLASS DEFINITION ==========================================================
class Vertex3D : public IOObject
{
public:
  Vertex3D(Mesh3D* mesh);
  Vertex3D(Mesh3D* mesh, const Point3Df& p);
  virtual ~Vertex3D();
  // calculate normal vector
  void update();
  // copy c'tor
  Vertex3D(const Vertex3D& other);

  // mutators
  void set(Vec3f p);
  void set(float x, float y, float z);

  // flags
  void setFlag(int flag, bool value);
  bool getFlag(int flag) const;
  bool boundary() const;
  void setBoundary(bool boundary);
  bool hidden() const;
  void setHidden(bool hidden);
  bool hasTexCoord() const;

  // ASCII stream IO
  virtual bool write(OutputHandler &oh) const;
  virtual bool read(InputHandler &ih);
  void write(FILE *fp) const;
  void read(FILE *fp);

  Mesh3D* mesh;
  Point3Df p;
  Vec3f n;
  Vec3uc c;
  int flags;
  std::vector<int> faces;

  //texture coordinates
  Vec2f t;
};

//==============================================================================
} // NAMESPACE rtc
//==============================================================================
#endif // VERTEX3D_H_ defined
//==============================================================================
