/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: VertexOctree.h
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 07/25/2006
 * modified ...: $Date: 2009-08-19 20:09:33 -0700 (Wed, 19 Aug 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 890 $
 */
#ifndef VERTEXOCTREE_H
#define VERTEXOCTREE_H

//== INCLUDES ==================================================================
#include <Geometry3D.h>
#include <mesh.h>
#include <vector>

//== NAMESPACES ================================================================
namespace puma {

//== CLASS DEFINITION ==========================================================
class VertexOctree
{
public:
  VertexOctree(
    const TriMesh*           mesh,
    unsigned int             maximum_depth,
    unsigned int             min_entries
    );

  VertexOctree(
    const TriMesh*           mesh,
    Vec3f                    center,
    float                    radius,
    unsigned int             maximum_depth,
    unsigned int             min_entries);

  VertexOctree(
    const TriMesh*           mesh,
    Vec3f                    center,
    float                    radius,
    unsigned int             current_depth,
    std::vector<VertexHandle>& vertices,
    unsigned int             maximum_depth,
    unsigned int             min_entries,
    int                      parentIndex = 0,
    VertexOctree*            parentTree = NULL);

  ~VertexOctree(void) {};

  bool searchNearest(const Vec3f &p, VertexHandle& vertex, float& distance);
  void refineTree();
  void getCubes(std::vector<Vec3f> &points, std::vector<int> &edges);
  void getCubeCenters(std::vector<Vec3f> &centers);
  void getPoints(std::vector<Vec3f> &points);
  void getVertices(std::vector<VertexHandle>& vertices);
  void getLeaves(std::vector<VertexOctree*>& leafes);

  Vec3f center() { return m_center; }
  float radius() { return m_radius; }
  bool isLeaf() { return m_isLeaf; }


private:
  void initialize(std::vector<VertexHandle>& faces);
  bool closerToVertex(const Vec3f &p, const VertexHandle& vertex_handle, float& distance);

private:
  const TriMesh*            m_mesh;
  Vec3f                     m_center;
  float                     m_radius;
  int                       m_parentIndex;
  VertexOctree*             m_parentTree;
  VertexOctree*             m_child[8];
  unsigned int              m_current_depth;

  // parameters
  unsigned int              m_maximum_depth;
  unsigned int              m_min_entries;

  bool                      m_isLeaf;
  std::vector<VertexHandle> m_vertices; // vertex indices, groups of three
};

//==============================================================================
} // NAMESPACE puma
//==============================================================================
#endif // VERTEXOCTREE_H defined
//==============================================================================
