/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcBinaryASCIIInputHandler.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */

//== INCLUDES ==================================================================
#include "rtc/rtcASCIIInputHandler.h"

//== NAMESPACES ================================================================
namespace rtc {

//== IMPLEMENTATION ============================================================

/// default constructor
ASCIIInputHandler::ASCIIInputHandler()
: InputHandler()
{
}

/// construct from istream
ASCIIInputHandler::ASCIIInputHandler(std::istream& is)
: InputHandler(is)
{
}

/// destructor
ASCIIInputHandler::~ASCIIInputHandler()
{
}

/// returns if the data format is binary or ascii
bool ASCIIInputHandler::binary() const
{
  return false;
}

/// read the name of a name-data pair
bool ASCIIInputHandler::readName(const std::string& name)
{
  if(!inStream) return false;
  std::string buf;
  // read the name
  *inStream >> buf;
  if (buf != name)
   return false;
  else
    return inStream->good();
}

/// read the seperator of a name-data pair
bool ASCIIInputHandler::readSeparator()
{
  return true;
}

bool ASCIIInputHandler::read(std::string& data) {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(double& data) {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(float& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(int& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(unsigned int& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(char& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(unsigned char& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(bool& data) {
  if(!inStream) return false;
  char tmp;
  ASCIIInputHandler::read(tmp);
  if (tmp == 0 || tmp == 1) {
    data=(tmp == 1);
    return inStream->good();
  } else {
    return false;
  }
}

bool ASCIIInputHandler::read(long& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(unsigned long& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(short& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(unsigned short& data)  {
  if(!inStream) return false;
  *inStream >> data;
  return inStream->good();
}

bool ASCIIInputHandler::read(char* data, long n) {
  if(!inStream) return false;
  inStream->read(data,n);
  return inStream->good();
}

//==============================================================================
} // namespace rtc
//==============================================================================
