/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcASCIIOutputHandler.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */

//== INCLUDES =================================================================
#include "rtc/rtcASCIIOutputHandler.h"

//== NAMESPACES ===============================================================
namespace rtc {

//== IMPLEMENTATION ==========================================================

/// default constructor
ASCIIOutputHandler::ASCIIOutputHandler()
: OutputHandler()
{
}

/// construct from ostream
ASCIIOutputHandler::ASCIIOutputHandler(std::ostream& os)
: OutputHandler(os)
{
}

/// destructor
ASCIIOutputHandler::~ASCIIOutputHandler()
{
}

/// returns if the data format is binary or ascii
bool ASCIIOutputHandler::binary() const
{
  return false;
}

/// write the name of a name-data pair
bool ASCIIOutputHandler::writeName(const std::string& name)
{
  if(!outStream) return false;
  *outStream << name;
  return outStream->good();
}
/// write the separator of a name-data pair
bool ASCIIOutputHandler::writeSeparator()
{
  if(!outStream) return false;
  *outStream << SEPARATOR;
  return outStream->good();
}

/// write the EOL symbol
bool ASCIIOutputHandler::writeEOL()
{
  if(!outStream) return false;
  *outStream << std::endl;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const std::string& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
};

bool ASCIIOutputHandler::write(const double& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const float& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const int& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const unsigned int& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const char& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const unsigned char& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const bool& data) {
  if(!outStream) return false;
  if (data) {
    return ASCIIOutputHandler::write(char(1));
  } else {
    return ASCIIOutputHandler::write(char(0));
  }
}

bool ASCIIOutputHandler::write(const long& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const unsigned long& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const short& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const unsigned short& data) {
  if(!outStream) return false;
  *outStream << data;
  return outStream->good();
}

bool ASCIIOutputHandler::write(const char* data, long n) {
  if(!outStream) return false;
  outStream->write(data,n);
  return outStream->good();
}

//=============================================================================
} // namespace rtc
//=============================================================================
