/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcBinaryBinaryInputHandler.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */

//== INCLUDES ==================================================================
#include "rtc/rtcBinaryInputHandler.h"
#include <vector>

//== NAMESPACES ================================================================
namespace rtc {

//== IMPLEMENTATION ============================================================

/// default constructor
BinaryInputHandler::BinaryInputHandler()
: InputHandler()
{
}

/// construct from istream
BinaryInputHandler::BinaryInputHandler(std::istream& is)
: InputHandler(is)
{
}

/// destructor
BinaryInputHandler::~BinaryInputHandler()
{
}

/// returns if the data format is binary or ascii
bool BinaryInputHandler::binary() const
{
  return true;
}

/// read the name of a name-data pair
bool BinaryInputHandler::readName(const std::string& name)
{
  return true;
}

/// read the seperator of a name-data pair
bool BinaryInputHandler::readSeparator()
{
  return true;
}

bool BinaryInputHandler::read(std::string& data) {
  if(!inStream) return false;
  size_t size;
  inStream->read(reinterpret_cast<char*>(&size),sizeof(size));
  if(size>0) {
    std::vector<char> tmp(size);
    inStream->read(&tmp[0], size); //deserialize characters of string
    data.assign(&tmp[0], size);
  }
  return inStream->good();
}

bool BinaryInputHandler::read(double& data) {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(float& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(int& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(unsigned int& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(char& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(unsigned char& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(bool& data) {
  if(!inStream) return false;
  char tmp;
  BinaryInputHandler::read(tmp);
  if (tmp == 0 || tmp == 1) {
    data=(tmp == 1);
    return inStream->good();
  } else {
    return false;
  }
}

bool BinaryInputHandler::read(long& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(unsigned long& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(short& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(unsigned short& data)  {
  if(!inStream) return false;
  inStream->read(reinterpret_cast<char*>(&data),sizeof(data));
  return inStream->good();
}

bool BinaryInputHandler::read(char* data, long n) {
  if(!inStream) return false;
  inStream->read(data,n);
  return inStream->good();
}

//==============================================================================
} // namespace rtc
//==============================================================================
