/*
 * Copyright (C) 2009
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcBinaryOutputHandler.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 01/20/2009
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */

//== INCLUDES =================================================================
#include "rtc/rtcBinaryOutputHandler.h"

//== NAMESPACES ===============================================================
namespace rtc {

//== IMPLEMENTATION ==========================================================

/// default constructor
BinaryOutputHandler::BinaryOutputHandler()
: OutputHandler()
{
}

/// construct from ostream
BinaryOutputHandler::BinaryOutputHandler(std::ostream& os)
: OutputHandler(os)
{
}

/// destructor
BinaryOutputHandler::~BinaryOutputHandler()
{
}

/// returns if the data format is binary or ascii
bool BinaryOutputHandler::binary() const
{
  return true;
}

/// write the name of a name-data pair
bool BinaryOutputHandler::writeName(const std::string& name)
{
  return true;
}
/// write the separator of a name-data pair
bool BinaryOutputHandler::writeSeparator()
{
  return true;
}

/// write the EOL symbol
bool BinaryOutputHandler::writeEOL()
{
  return true;
}

bool BinaryOutputHandler::write(const std::string& data) {
  if(!outStream) return false;
  size_t size = data.size();
  //  *outStream << size;
  outStream->write(reinterpret_cast<const char*>(&size),sizeof(size));
 
  if(size>0) {
    *outStream << data;
    //alternative:
    //outStream->write(data.c_str(),size);
  }
  return outStream->good();
};

bool BinaryOutputHandler::write(const double& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const float& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const int& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const unsigned int& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const char& data) {
  if(!outStream) return false;
  outStream->put(data);
  return outStream->good();
}

bool BinaryOutputHandler::write(const unsigned char& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const bool& data) {
  if(!outStream) return false;
  if (data) {
    return BinaryOutputHandler::write(char(1));
  } else {
    return BinaryOutputHandler::write(char(0));
  }
}

bool BinaryOutputHandler::write(const long& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const unsigned long& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const short& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const unsigned short& data) {
  if(!outStream) return false;
  outStream->write(reinterpret_cast<const char*>(&data),sizeof(data));
  return outStream->good();
}

bool BinaryOutputHandler::write(const char* data, long n) {
  if(!outStream) return false;
  outStream->write(data,n);
  return outStream->good();
}

//=============================================================================
} // namespace rtc
//=============================================================================
