/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: BoundingBox.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 08/03/2006
 * modified ...: $Date:2008-06-02 19:31:25 -0700 (Mon, 02 Jun 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:298 $
 */
#ifdef _MSC_VER
#  pragma warning(disable: 4244 4267 4311 4305)
#endif

//== INCLUDES ==================================================================
#include "BoundingBox.h"

//== NAMESPACES ================================================================
namespace rtc {

//== IMPLEMENTATION ============================================================
bool BoundingBox::apply(const TriMesh& mesh, Point& bbMin, Point& bbMax)
{
  TriMesh::ConstVertexIter vIt(mesh.vertices_begin());
  TriMesh::ConstVertexIter vEnd(mesh.vertices_end());

  bbMin = bbMax = mesh.point(vIt);

  for (;vIt!=vEnd; ++vIt)
  {
    bbMin.minimize(mesh.point(vIt));
    bbMax.maximize(mesh.point(vIt));
  }
  return true;
}

bool BoundingBox::apply(const std::vector<TriMesh*> meshes, Point& bbMin, Point& bbMax)
{
  std::vector<TriMesh*>::const_iterator it, it_end;
  for(it=meshes.begin(), it_end=meshes.end(); it!=it_end; ++it)
  {
    Point bbMin_tmp, bbMax_tmp;
    const TriMesh* mesh = *it;
    apply(*mesh,bbMin_tmp,bbMax_tmp);
    if(it!=meshes.begin())
    {
      bbMin.minimize(bbMin_tmp);
      bbMax.maximize(bbMax_tmp);
    }
    else
    {
      bbMin=bbMin_tmp;
      bbMax=bbMax_tmp;
    }
  }
  return true;
}

//=============================================================================
} // namespace Merging
//=============================================================================
