/*
 * Copyright (C) 2007
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: Face3D.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 03/28/2008
 * modified ...: $Date:2008-03-03 10:26:02 -0800 (Mon, 03 Mar 2008) $
 * changed by .: $Author:benjaminpitzer $
 * revision ...: $Revision:141 $
 */

//== INCLUDES ==================================================================
#include "rtc/rtcGeometry3D.h"
#include "rtc/rtcMesh3D.h"
#include "rtc/rtcVertex3D.h"
#include "rtc/rtcFace3D.h"

#define FACE_FLAG_BOUNDARY    0x01
#define FACE_FLAG_HIDDEN      0x02
#define FACE_FLAG_EMPTY2      0x04
#define FACE_FLAG_EMPTY3      0x08
#define FACE_FLAG_EMPTY4      0x10
#define FACE_FLAG_EMPTY5      0x20
#define FACE_FLAG_EMPTY6      0x40
#define FACE_FLAG_EMPTY7      0x80

//==============================================================================
// NAMESPACE
namespace rtc {

Face3D::Face3D(Mesh3D* _mesh, int v0, int v1, int v2)
:mesh(_mesh),v(v0,v1,v2),flags(0)
{
}

Face3D::~Face3D()
{
}

Face3D::Face3D(const Face3D& other)
{
  mesh = other.mesh;
  v = other.v;
  n = other.n;
  flags = other.flags;
}

// calculate edge vector and normal vector
void Face3D::update()
{
  Vec3f t1 = mesh->vertices[v(1)]->p-mesh->vertices[v(0)]->p;
  Vec3f t2 = mesh->vertices[v(2)]->p-mesh->vertices[v(0)]->p;
  n = t1.cross(t2);
  n.normalize();
}

bool Face3D::getFlag(int flag) const
{
  return ((flags&flag)!=0);
}

void Face3D::setFlag(int flag, bool value)
{
  if(value)
    flags|=flag;
  else
    flags&=flag;
}

bool Face3D::boundary() const
{
  return getFlag(FACE_FLAG_BOUNDARY);
}

void Face3D::setBoundary(bool boundary)
{
  setFlag(FACE_FLAG_BOUNDARY,boundary);
}

bool Face3D::hidden() const
{
  return getFlag(FACE_FLAG_HIDDEN);
}

void Face3D::setHidden(bool hidden)
{
  setFlag(FACE_FLAG_HIDDEN,hidden);
}

bool Face3D::write(OutputHandler &oh) const
{
  rtc_write(oh,v);
  rtc_write(oh,n);
  rtc_write(oh,"flags",flags);
  return true;
}

void Face3D::write( FILE *fp ) const
{
  fwrite(&v,sizeof(v),1,fp);
  fwrite(&n,sizeof(n),1,fp);
  fwrite(&flags,sizeof(flags),1,fp);
}

bool Face3D::read(InputHandler &ih)
{
  rtc_read(ih,v);
  rtc_read(ih,n);
  rtc_read(ih,"flags",flags);
  return true;
}

void Face3D::read( FILE *fp )
{
  size_t res = 0;
  res+=fread(&v,sizeof(v),1,fp);
  res+=fread(&n,sizeof(n),1,fp);
  res+=fread(&flags,sizeof(flags),1,fp);
}

//==============================================================================
} // NAMESPACE rtc
//==============================================================================
