/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: PUMA: Probablistic Unsupervised Model Acquisition
 * file .......: Functor.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 08/16/2006
 * modified ...: $Date: 2008-05-26 18:51:04 -0700 (Mon, 26 May 2008) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 288 $
 */

//== INCLUDES ==================================================================
#include "rtc/rtcFunctor.h"

//== NAMESPACES ================================================================
namespace rtc {

  //== IMPLEMENTATION ==========================================================

  // ----------------------------
  // Parameters
  // ----------------------------
  Functor::Parameters::Parameters()
  : IOObject()
  {
  }

  Functor::Parameters::Parameters(const Parameters& other)
  : IOObject()
  {
    copy(other);
  }


  Functor::Parameters::~Parameters()
  {
  }

  Functor::Parameters& Functor::Parameters::copy(const Parameters& other) {
    return (*this);
  }

  Functor::Parameters& Functor::Parameters::operator=(const Parameters& other){
    return (*this);
  }

  /*
  * read the functor parameters
  */
  bool Functor::Parameters::read(InputHandler &ih)
  {
    return true;
  }

  /*
  * write the functor parameters
  */
  bool Functor::Parameters::write(OutputHandler &oh) const
  {
    return true;
  }


  // ----------------------------
  // Functor
  // ----------------------------

  // constructor
  Functor::Functor()
  : IOObject(),params(0)
  {
  }

  // copy constructor
  Functor::Functor(const Functor& other)
  : IOObject(),params(0)
  {
    copy(other);
  }

  // destructor
  Functor::~Functor()
  {
    delete params;
  }

  // copy constructor

  /* sets the Functor's Parameters.
     The Functor keeps its own copy of the given Parameters.*/
  bool Functor::setParameters(const Functor::Parameters& theParams)
  {
    delete params;
    params = 0;
    params = theParams.clone();
    return true;
  }

  /*
   * returns current Parameters.
   */
  const Functor::Parameters& Functor::getParameters() const
  {
    return *params;
  }

  // copy data of "other" Functor.
  Functor& Functor::copy(const Functor& other)
  {
    setParameters(other.getParameters());
    return (*this);
  }

  // alias for copy
  Functor& Functor::operator=(const Functor& other) {
    return (*this);
  }

  /**
  * Adds a status message to the messages
  */
  void Functor::addStatusMessage(const StatusMessage msg) {
    m_messages.push_back(msg);
  }

  /**
  * Returns the list of all status messages
  */
  const Functor::StatusMessages& Functor::getStatusMessages() const {
    return m_messages;
  }

  /*
  * read the functor parameters
  */
  bool Functor::read(InputHandler &ih)
  {
    if (params != 0) {
      return params->read(ih);
    } else {
      addStatusMessage("Tried to read <null> parameters object");
      return false;
    }
  }

  /*
  * read the functor parameters
  */
  bool Functor::write(OutputHandler &oh) const
  {
    if (params != 0) {
      return params->write(oh);
    } else {
      return false;
    }
  }

//==============================================================================
} // namespace rtc
//==============================================================================
