/*
 * Copyright (C) 2008
 * Robert Bosch LLC
 * Research and Technology Center North America
 * Palo Alto, California
 *
 * All rights reserved.
 *
 *------------------------------------------------------------------------------
 * project ....: Autonomous Technologies
 * file .......: rtcStringTools.cpp
 * authors ....: Benjamin Pitzer
 * organization: Robert Bosch LLC
 * creation ...: 04/16/2006
 * modified ...: $Date: 2009-01-21 18:19:16 -0800 (Wed, 21 Jan 2009) $
 * changed by .: $Author: benjaminpitzer $
 * revision ...: $Revision: 14 $
 */
#ifdef _MSC_VER
#  pragma warning(disable: 4244)
#endif

//== INCLUDES ==================================================================
#include "rtc/rtcStringTools.h"
#include <stdio.h>
#include <string.h>
#include <sstream>
#include <iostream>
#include <boost/lexical_cast.hpp>

//== NAMESPACES ================================================================
namespace rtc {

bool lexical_cast_bool(const std::string& str, const bool bDefault)
{
  if (strcmp(str.c_str(),"true")==0 ||
      strcmp(str.c_str(),"True")==0 ||
      strcmp(str.c_str(),"TRUE")==0 ||
      strcmp(str.c_str(),"1")==0 ||
      strcmp(str.c_str(),"-1")==0)
    return true;
  else if (
      strcmp(str.c_str(),"false")==0 ||
      strcmp(str.c_str(),"False")==0 ||
      strcmp(str.c_str(),"FALSE")==0 ||
      strcmp(str.c_str(),"0")==0)
    return false;
  else
    return bDefault;
}

int lexical_cast_int(const std::string& str, const int nDefault)
{
  int value;
  int res;
  if(str.length() == 0)
    return nDefault;
  res=sscanf(str.c_str(),"%i",&value);
  if (res==1)
    return value;
  return 0;
}

float lexical_cast_float(const std::string& str, const float fDefault)
{
  double value;
  try { value = boost::lexical_cast<float>(str); }
  catch(boost::bad_lexical_cast &) { value = fDefault; }
  return value;
}

double lexical_cast_double(const std::string& str, const double dDefault )
{
  double value;
  try { value = boost::lexical_cast<double>(str); }
  catch(boost::bad_lexical_cast &) { value = dDefault; }
  return value;
}

long lexical_cas_long(const std::string& str, const long lDefault )
{
  int value;
  int res;
  if(str.length() == 0)
    return lDefault;
  res=sscanf(str.c_str(),"%i",&value);
  if (res!=1)
    return lDefault;
  return (long)value;
}

std::string lexical_cast_string(const double& var)
{
  std::stringstream ss;
  ss << var;
  return ss.str();
}

std::string lexical_cast_string(const float& var)
{
  std::stringstream ss;
  ss << var;
  return ss.str();
}

std::string lexical_cast_string(const bool& var)
{
  if(var)
    return std::string("true");
  else
    return std::string("false");
}

std::string lexical_cast_string(const int& var){
  std::stringstream ss;
  ss << var;
  return ss.str();
}

void split_string(const std::string& str, std::vector<std::string>& tokens, const std::string& delimiters)
{
  // Skip delimiters at beginning.
  std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
  // Find first "non-delimiter".
  std::string::size_type pos     = str.find_first_of(delimiters, lastPos);

  while (std::string::npos != pos || std::string::npos != lastPos)
  {
    // Found a token, add it to the vector.
    tokens.push_back(str.substr(lastPos, pos - lastPos));
    // Skip delimiters.  Note the "not_of"
    lastPos = str.find_first_not_of(delimiters, pos);
    // Find next "non-delimiter"
    pos = str.find_first_of(delimiters, lastPos);
  }
}

std::string& trim(std::string& str)
{
  // Trim Both leading and trailing spaces
  size_t startpos = str.find_first_not_of(" \t"); // Find the first character position after excluding leading blank spaces
  size_t endpos = str.find_last_not_of(" \t"); // Find the first character position from reverse af
  // if all spaces or empty return an empty string
  if(( std::string::npos == startpos ) || ( std::string::npos == endpos))
    str = "";
  else
    str = str.substr(startpos, endpos-startpos+1);
  return str;
}

//==============================================================================
} // NAMESPACE rtc
//==============================================================================
